#include"../machine/opwolf.c"
/****************************************************************************

Operation Wolf  (c) Taito 1987
==============

David Graves, Jarek Burczynski

Sources:		MAME Rastan driver
			MAME Taito F2 driver
			Raine source - many thanks to Richard Bush
			  and the Raine Team.

Main CPU: MC68000 uses irq 5.
Sound   : Z80 & YM2151 & MSM5205


Operation Wolf uses similar hardware to Rainbow Islands and Rastan.
The screen layout and registers and sprites appear to be identical.

The original game has a c-chip like Rainbow. To emulate this we
follow Raine and emulate the extra Z80 from the *bootleg* board,
which acts as a substitute c-chip. $800 bytes of the Z80's ram
space are shared, mapped to $800 words in the 68000 address space
used to access the c-chip.


Gun Travel
----------

Horizontal gun travel maybe could be widened to include more
of the status bar (you can shoot enemies underneath it).

To keep the input span 0-255 a multiplier (300/256 ?)
would be used.


TODO
====

Need to verify Opwolf against original board: various reports
claim there are discrepancies (perhaps limitations of the fake
Z80 c-chip substitute to blame?).

There are a few unmapped writes for the sound Z80 in the log.

What number should be returned for the c-chip Z80 interrupt?

Raine source has standard Asuka/Mofflot sprite/tile priority:
0x2000 in sprite_ctrl puts all sprites under top bg layer. But
Raine simply kludges in this value, failing to read it from a
register. So what is controlling priority.


***************************************************************************/
/* #define NEWSOUND 1 to use MSM5205 instead of ADPCM */
/* Currently the MSM5205 sound is slow and bad */
#define NEWSOUND 0

#include "driver.h"
#include "state.h"
#include "cpu/m68000/m68000.h"
#include "vidhrdw/generic.h"
#include "vidhrdw/taitoic.h"
#include "sndhrdw/taitosnd.h"

void rainbow_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh);
void rastan_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh);

extern unsigned char *rastan_videoram1,*rastan_videoram3;
extern size_t rastan_videoram_size;
extern unsigned char *rastan_spriteram;
extern unsigned char *rastan_scrollx;
extern unsigned char *rastan_scrolly;

WRITE_HANDLER( rastan_videoram1_w );
READ_HANDLER( rastan_videoram1_r );
WRITE_HANDLER( rastan_videoram3_w );
READ_HANDLER( rastan_videoram3_r );

WRITE_HANDLER( rastan_scrollY_w );
WRITE_HANDLER( rastan_scrollX_w );

WRITE_HANDLER( rainbow_spritectrl_w );
WRITE_HANDLER( rastan_flipscreen_w );


static UINT8 adpcm_b[0x08];
static UINT8 adpcm_c[0x08];
static int opwolf_gun_xoffs,opwolf_gun_yoffs;

void opwolf_cchip_init(void);
READ_HANDLER( opwolf_cchip_status_r );
READ_HANDLER( opwolf_cchip_data_r );
WRITE_HANDLER( opwolf_cchip_status_w );
WRITE_HANDLER( opwolf_cchip_data_w );
WRITE_HANDLER( opwolf_cchip_bank_w );

WRITE_HANDLER( rainbow_spritectrl_w );
WRITE_HANDLER( rastan_spriteflip_w );

int  opwolf_vh_start(void);
void opwolf_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh);
void opwolf_vh_stop(void);


/***********************************************************
				INTERRUPTS
***********************************************************/

//void opwolf_irq_handler(int irq);

static int opwolf_interrupt(void)
{
	return 5;  /* interrupt vector 5: others are RTE */
}


/**********************************************************
				GAME INPUTS
**********************************************************/
static READ_HANDLER( opwolf_lightgun_r )
{
	int scaled;
	switch (offset)
	{
	    case 0x00:      /* P1X - Have to remap 8 bit input value, into 0-319 visible range */
		scaled=(input_port_5_r(0) * 320 ) >> 8;
		return (scaled + 0x15 + opwolf_gun_xoffs);
	    case 0x02:      /* P1Y */
		return (input_port_6_r(0) - 0x24 + opwolf_gun_yoffs);
	}

	return 0xff;
}

/******************************************************
				SOUND
******************************************************/

static WRITE_HANDLER( sound_bankswitch_w )
{
	int banknum = (data-1) & 0x03;
        // 10 matches MRA_BANK10 in Z80 MemoryReadAddress
	cpu_setbank( 10, memory_region(REGION_CPU2) + (banknum * 0x4000) + 0x10000 );
}

static READ_HANDLER( opwolf_sound_r )
{
	return taitosound_comm_r(0) << 8;
}

static WRITE_HANDLER( opwolf_sound_w )
{
	if (offset == 0)
	{
	    taitosound_port_w(0,data >> 8);
	}
	else if (offset == 2)
	{
	    taitosound_comm_w(0,data >> 8);
	}
}



/***********************************************************
			 MEMORY STRUCTURES
***********************************************************/

static struct MemoryReadAddress opwolf_readmem[] =
{
	{ 0x000000, 0x03ffff, MRA_ROM },
	{ 0x0f0000, 0x0f07ff, opwolf_cchip_data_r },
	{ 0x0f0802, 0x0f0803, opwolf_cchip_status_r },
	{ 0x0ff000, 0x0ff7ff, opwolf_cchip_data_r },	    /* mirror */
	{ 0x0ff802, 0x0ff803, opwolf_cchip_status_r },	    /* mirror */
	{ 0x100000, 0x107fff, MRA_BANK1 },	/* RAM */
	{ 0x200000, 0x200fff, paletteram_word_r },
	{ 0x380000, 0x380001, input_port_2_r },	/* DSW A */
	{ 0x380002, 0x380003, input_port_3_r },	/* DSW B */
	{ 0x3a0000, 0x3a0003, opwolf_lightgun_r },	/* lightgun, read at $11e0/6 */
	{ 0x3e0000, 0x3e0001, MRA_NOP },
	{ 0x3e0002, 0x3e0003, opwolf_sound_r },
	//{ 0x3e0002, 0x3e0003, taitosound_comm16_msb_r },
	{ 0xc00000, 0xc0ffff, PC080SN_word_0_r },
	{ 0xd00000, 0xd007ff, MRA_BANK5 },
	{ -1 }  /* end of table */
};


static struct MemoryWriteAddress opwolf_writemem[] =
{
	{ 0x000000, 0x03ffff, MWA_ROM },
	//{ 0x0ff000, 0x0fffff, cchip_w },
	{ 0x0ff000, 0x0ff7ff, opwolf_cchip_data_w },
	{ 0x0ff802, 0x0ff803, opwolf_cchip_status_w },
	{ 0x0ffc00, 0x0ffc01, opwolf_cchip_bank_w },
	{ 0x100000, 0x107fff, MWA_BANK1 },
	{ 0x200000, 0x200fff, paletteram_xxxxRRRRGGGGBBBB_word_w, &paletteram },
	{ 0x380000, 0x380003, rainbow_spritectrl_w },	// usually 0x4, changes when you fire
	{ 0x3c0000, 0x3c0001, MWA_NOP },	/* watchdog ?? */
	{ 0x3e0000, 0x3e0003, opwolf_sound_w },
	//{ 0x3e0000, 0x3e0001, taitosound_port16_msb_w },
	//{ 0x3e0002, 0x3e0003, taitosound_comm16_msb_w },
	{ 0xc00000, 0xc0ffff, PC080SN_word_0_w },
	{ 0xc10000, 0xc1ffff, MWA_BANK2 },      /* error in init code (?) */
	{ 0xc20000, 0xc20003, PC080SN_yscroll_word_0_w },
	{ 0xc40000, 0xc40003, PC080SN_xscroll_word_0_w },
	{ 0xc50000, 0xc50003, PC080SN_ctrl_word_0_w },
	{ 0xd00000, 0xd007ff, MWA_BANK5, &rastan_spriteram, &spriteram_size },
	{ 0xd01bfe, 0xd01bff, rastan_spriteflip_w },
	{ -1 }  /* end of table */
};

/***************************************************************************/

static struct MemoryReadAddress z80_readmem[] =
{
	{ 0x0000, 0x3fff, MRA_ROM },
	{ 0x4000, 0x7fff, MRA_BANK10 },
	{ 0x8000, 0x8fff, MRA_RAM },
	{ 0x9001, 0x9001, YM2151_status_port_0_r },
	{ 0x9002, 0x9100, MRA_RAM },
	{ 0xa001, 0xa001, taitosound_slave_comm_r },
	{ -1 }  /* end of table */
};



//static unsigned char adpcm_d[0x08];
//0 - start ROM offset LSB
//1 - start ROM offset MSB
//2 - end ROM offset LSB
//3 - end ROM offset MSB
//start & end need to be multiplied by 16 to get a proper _byte_ address in adpcm ROM
//4 - always zero write (start trigger ?)
//5 - different values
//6 - different values
//

#if NEWSOUND
static UINT32 adpcm_pos[2],adpcm_end[2];
static void opwolf_msm5205_vck(int chip)
{
	static int adpcm_data[2] = { -1, -1 };

	if (adpcm_data[chip] != -1)
	{
	    MSM5205_data_w(chip, adpcm_data[chip] & 0x0f);
	    adpcm_data[chip] = -1;
	    if (adpcm_pos[chip] == adpcm_end[chip])
	    {
		MSM5205_reset_w(chip, 1);
	    }
	}
	else
	{
	    adpcm_data[chip] = memory_region(REGION_SOUND1)[adpcm_pos[chip]];
	    adpcm_pos[chip] = (adpcm_pos[chip] + 1) & 0x7ffff;
	    MSM5205_data_w(chip, adpcm_data[chip] >> 4);
	}
}
#endif


static WRITE_HANDLER( opwolf_adpcm_b_w )
{
	int start;
	int end;

	adpcm_b[offset] = data;

	if (offset==0x04) //trigger ?
	{
		start = adpcm_b[0] + adpcm_b[1]*256;
		end   = adpcm_b[2] + adpcm_b[3]*256;
		start *=16;
		end   *=16;
#if NEWSOUND
		adpcm_pos[0] = start;
		adpcm_end[0] = end;
		MSM5205_reset_w(0, 0);
#else
		ADPCM_play(0,start,(end-start)*2);
#endif
	}
}


static WRITE_HANDLER( opwolf_adpcm_c_w )
{
	int start;
	int end;

	adpcm_c[offset] = data;

	if (offset==0x04) //trigger ?
	{
		start = adpcm_c[0] + adpcm_c[1]*256;
		end   = adpcm_c[2] + adpcm_c[3]*256;
		start *=16;
		end   *=16;
#if NEWSOUND
		adpcm_pos[1] = start;
		adpcm_end[1] = end;
		MSM5205_reset_w(1, 0);
#else
		ADPCM_play(1,start,(end-start)*2);
#endif
	}
}


static WRITE_HANDLER( opwolf_adpcm_d_w )
{
}

static WRITE_HANDLER( opwolf_adpcm_e_w )
{
}


static struct MemoryWriteAddress z80_writemem[] =
{
	{ 0x0000, 0x7fff, MWA_ROM },
	{ 0x8000, 0x8fff, MWA_RAM },
	{ 0x9000, 0x9000, YM2151_register_port_0_w },
	{ 0x9001, 0x9001, YM2151_data_port_0_w },
	{ 0xa000, 0xa000, taitosound_slave_port_w },
	{ 0xa001, 0xa001, taitosound_slave_comm_w },
	{ 0xb000, 0xb006, opwolf_adpcm_b_w },
	{ 0xc000, 0xc006, opwolf_adpcm_c_w },
	{ 0xd000, 0xd000, opwolf_adpcm_d_w },
	{ 0xe000, 0xe000, opwolf_adpcm_e_w },
	{ -1 }  /* end of table */
};


/***********************************************************
			 INPUT PORTS, DIPs
***********************************************************/

#define TAITO_COINAGE_WORLD_8 \
	PORT_DIPNAME( 0x30, 0x30, DEF_STR( Coin_A ) ) \
	PORT_DIPSETTING(    0x00, DEF_STR( 4C_1C ) ) \
	PORT_DIPSETTING(    0x10, DEF_STR( 3C_1C ) ) \
	PORT_DIPSETTING(    0x20, DEF_STR( 2C_1C ) ) \
	PORT_DIPSETTING(    0x30, DEF_STR( 1C_1C ) ) \
	PORT_DIPNAME( 0xc0, 0xc0, DEF_STR( Coin_B ) ) \
	PORT_DIPSETTING(    0xc0, DEF_STR( 1C_2C ) ) \
	PORT_DIPSETTING(    0x80, DEF_STR( 1C_3C ) ) \
	PORT_DIPSETTING(    0x40, DEF_STR( 1C_4C ) ) \
	PORT_DIPSETTING(    0x00, DEF_STR( 1C_6C ) )

#define TAITO_DIFFICULTY_8 \
	PORT_DIPNAME( 0x03, 0x03, DEF_STR( Difficulty ) ) \
	PORT_DIPSETTING(    0x02, "Easy" ) \
	PORT_DIPSETTING(    0x03, "Medium" ) \
	PORT_DIPSETTING(    0x01, "Hard" ) \
	PORT_DIPSETTING(    0x00, "Hardest" )

INPUT_PORTS_START( opwolf )
	PORT_START	/* IN0 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_COIN1 )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_COIN2 )
	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x20, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x40, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x80, IP_ACTIVE_LOW,  IPT_UNKNOWN )

	PORT_START	/* IN1 */
	PORT_BIT( 0x01, IP_ACTIVE_LOW,  IPT_BUTTON1 )
	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_BUTTON2 )
	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_SERVICE1 )
	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_TILT )
	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_START1 )
	PORT_BIT( 0x20, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x40, IP_ACTIVE_LOW,  IPT_UNKNOWN )
	PORT_BIT( 0x80, IP_ACTIVE_LOW,  IPT_UNKNOWN )

	PORT_START /* DSW A */
	PORT_DIPNAME( 0x01, 0x01, "NY Conversion of Upright" )
	PORT_DIPSETTING(    0x01, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x02, 0x00, "Allow Continue" )
	PORT_DIPSETTING(    0x02, DEF_STR( No ))
	PORT_DIPSETTING(    0x00, DEF_STR( Yes ))
	PORT_SERVICE( 0x04, IP_ACTIVE_LOW )
	PORT_DIPNAME( 0x08, 0x08, DEF_STR( Demo_Sounds ) )
	PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x08, DEF_STR( On ) )
	TAITO_COINAGE_WORLD_8

	PORT_START /* DSW B */
	TAITO_DIFFICULTY_8
	PORT_DIPNAME( 0x0c, 0x0c, "Ammo Magazines at Start" )
	PORT_DIPSETTING(    0x00, "4" )
	PORT_DIPSETTING(    0x04, "5" )
	PORT_DIPSETTING(    0x0c, "6" )
	PORT_DIPSETTING(    0x08, "7" )
	PORT_DIPNAME( 0x10, 0x10, DEF_STR( Unused ) )	// Manual says all 3 unused
	PORT_DIPSETTING(    0x10, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x20, 0x20, DEF_STR( Unused ) )
	PORT_DIPSETTING(    0x20, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x40, 0x40, DEF_STR( Unused ) )
	PORT_DIPSETTING(    0x40, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x80, 0x00, "Language" )
	PORT_DIPSETTING(    0x80, "Japanese" )
	PORT_DIPSETTING(    0x00, "English" )

	PORT_START	/* Fake DSW */
	PORT_BITX(    0x01, 0x01, IPT_DIPSWITCH_NAME | IPF_TOGGLE, "Show gun target", IP_KEY_NONE, IP_JOY_NONE )
	PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x01, DEF_STR( On ) )

	PORT_START	/* P1X (span allows you to shoot enemies behind status bar) */
	//PORT_ANALOG( 0xff, 0x80, IPT_AD_STICK_X | IPF_PLAYER1, 25, 15, 0x00, 0xff)
	PORT_ANALOG( 0xff, 0x80, IPT_LIGHTGUN_X| IPF_PLAYER1, 25, 15, 0, 255 )

	PORT_START	/* P1Y (span allows you to be slightly offscreen) */
	//PORT_ANALOG( 0xff, 0x80, IPT_AD_STICK_Y | IPF_PLAYER1, 25, 15, 0x00, 0xff)
	PORT_ANALOG( 0xff, 0x80, IPT_LIGHTGUN_Y| IPF_PLAYER1, 25, 15, 0, 255 )
INPUT_PORTS_END


/**************************************************************
				GFX DECODING
**************************************************************/

static struct GfxLayout charlayout =
{
	8,8,	/* 8*8 characters */
	RGN_FRAC(1,1),
	4,	/* 4 bits per pixel */
	{ 0, 1, 2, 3 },
	{ 2*4, 3*4, 0*4, 1*4, 6*4, 7*4, 4*4, 5*4 },
	{ 0*32, 1*32, 2*32, 3*32, 4*32, 5*32, 6*32, 7*32 },
	32*8	/* every sprite takes 32 consecutive bytes */
};

static struct GfxLayout tilelayout =
{
	16,16,	/* 16*16 sprites */
	RGN_FRAC(1,1),
	4,	/* 4 bits per pixel */
	{ 0, 1, 2, 3 },
	{ 2*4, 3*4, 0*4, 1*4, 6*4, 7*4, 4*4, 5*4, 10*4, 11*4, 8*4, 9*4, 14*4, 15*4, 12*4, 13*4 },
	{ 0*64, 1*64, 2*64, 3*64, 4*64, 5*64, 6*64, 7*64, 8*64, 9*64, 10*64, 11*64, 12*64, 13*64, 14*64, 15*64 },
	128*8	/* every sprite takes 128 consecutive bytes */
};

static struct GfxDecodeInfo opwolf_gfxdecodeinfo[] =
{
	{ REGION_GFX2, 0, &tilelayout,  0, 256 },	/* sprites */
	{ REGION_GFX1, 0, &charlayout,  0, 256 },	/* scr tiles */
	{ -1 } /* end of array */
};


/**************************************************************
			     YM2151 (SOUND)
**************************************************************/

/* handler called by the YM2151 emulator when the internal timers cause an IRQ */

static void irq_handler(int irq)
{
	cpu_set_irq_line(1,0,irq ? ASSERT_LINE : CLEAR_LINE);
}


static struct YM2151interface ym2151_interface =
{
	1,			/* 1 chip */
	4000000,	/* 4 MHz ? */
	{ YM3012_VOL(50,MIXER_PAN_CENTER,50,MIXER_PAN_CENTER) },
	{ irq_handler },
	{ sound_bankswitch_w }
};

#if NEWSOUND
static struct MSM5205interface msm5205_interface =
{
	2,                     /* 2 chips       */
	384000,                /* 384KHz        */
	{ opwolf_msm5205_vck, opwolf_msm5205_vck },/* irq function  */
	{ MSM5205_S48_4B, MSM5205_S48_4B}, /* 8 KHz */
	{ 60, 60 }                 /* volume        */
};
#else
static struct ADPCMinterface adpcm_interface =
{
	2,		/* 2 chips ?? */
	8000,		/* 8000Hz playback */
	REGION_SOUND1,	/* memory region */
	0,
	{ 60, 60 }	/* volume  */
};
#endif


/***********************************************************
			     MACHINE DRIVERS
***********************************************************/

static struct MachineDriver machine_driver_opwolf =
{
	/* basic machine hardware */
	{
		{
			CPU_M68000,
			8000000,	/* 8 MHz */
			opwolf_readmem,opwolf_writemem,0,0,
			opwolf_interrupt,1
		},
		{
			CPU_Z80,
			4000000,	/* 4 MHz */
			z80_readmem,z80_writemem,0,0,
			ignore_interrupt,1
		}
	},
	60, DEFAULT_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
	10,	/* 10 CPU slices per frame - enough for the sound CPU to read all commands */
	0,

	/* video hardware */
	40*8, 32*8, { 0*8, 40*8-1, 1*8, 31*8-1 },
	opwolf_gfxdecodeinfo,
	8192, 8192,
	0,

	VIDEO_TYPE_RASTER | VIDEO_MODIFIES_PALETTE,
	0,
	opwolf_vh_start,
	opwolf_vh_stop,
	opwolf_vh_screenrefresh,

	/* sound hardware */
	SOUND_SUPPORTS_STEREO,0,0,0,
	{
		{
			SOUND_YM2151,
			&ym2151_interface
		},
#if NEWSOUND
		{
			SOUND_MSM5205,
			&msm5205_interface
		}
#else
		{
			SOUND_ADPCM,
			&adpcm_interface
		}
#endif
	}
};


/***************************************************************************
					DRIVERS
***************************************************************************/

ROM_START( opwolf )
	ROM_REGION( 0x40000, REGION_CPU1 )     /* 256k for 68000 code */
	ROM_LOAD_EVEN( "b20-05-02.40",  0x00000, 0x10000, 0x3ffbfe3a )
	ROM_LOAD_ODD ( "b20-03-02.30",  0x00000, 0x10000, 0xfdabd8a5 )
	ROM_LOAD_EVEN( "b20-04.39",  0x20000, 0x10000, 0x216b4838 )
	ROM_LOAD_ODD ( "b20-20.29",  0x20000, 0x10000, 0xd244431a )

	ROM_REGION( 0x20000, REGION_CPU2 )      /* sound cpu */
	ROM_LOAD( "b20-07.10",  0x00000, 0x04000, 0x45c7ace3 )
	ROM_CONTINUE(            0x10000, 0x0c000 ) /* banked stuff */

	ROM_REGION( 0x80000, REGION_GFX1 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "b20-13.13",  0x00000, 0x80000, 0xf6acdab1 )	/* SCR tiles (8 x 8) */

	ROM_REGION( 0x80000, REGION_GFX2 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "b20-14.72",  0x00000, 0x80000, 0x89f889e5 )	/* Sprites (16 x 16) */

	ROM_REGION( 0x80000, REGION_SOUND1 )	/* ADPCM samples */
	ROM_LOAD( "b20-08.21",   0x00000, 0x80000, 0xf3e19c64 )
ROM_END

static void init_opwolf(void)
{

	opwolf_cchip_init();

	// World & US version have different gun offsets, presumably slightly different gun hardware
	//unsigned char* rom=memory_region(REGION_CPU1);
	//opwolf_gun_xoffs = 0xec - (rom[0x1ffd8]&0xff);
	//opwolf_gun_yoffs = 0x1c - (rom[0x1ffd7]&0xff);
	//
	//The above values don't work very well however these do (from opbear).
	//
	opwolf_gun_xoffs = -2;
	opwolf_gun_yoffs = 17;
}

/*    year  rom       parent    machine   inp       init */
GAME( 1987, opwolf,   0,        opwolf,   opwolf,   opwolf,   ROT0, "Taito America Corp", "Operation Wolf (US)" )
