/*##########################################################################

	atarigen.c

	General functions for Atari raster games.

##########################################################################*/


#include "driver.h"
#include "atarigen.h"
#include "slapstic.h"



/*##########################################################################
	STATIC VARIABLES
##########################################################################*/

static UINT8 		atarigen_slapstic59_num;
static data16_t *	atarigen_slapstic59;
static int		atarigen_slapstic59_bank;
static data8_t 		atarigen_slapstic59_bank0[0x2000];


/*##########################################################################
	SLAPSTIC HANDLING
##########################################################################*/

inline void update_bank(int bank)
{
	/* if the bank has changed, copy the memory; Pit Fighter needs this */
	if (bank != atarigen_slapstic59_bank)
	{
		/* bank 0 comes from the copy we made earlier */
		if (bank == 0)
			memcpy(atarigen_slapstic59, atarigen_slapstic59_bank0, 0x2000);
		else
			memcpy(atarigen_slapstic59, &atarigen_slapstic59[bank * 0x1000], 0x2000);

		/* remember the current bank */
		atarigen_slapstic59_bank = bank;
	}
}


/*---------------------------------------------------------------
	atarigen_slapstic59_w: Assuming that the slapstic59 sits in
	ROM memory space, we just simply tweak the slapstic59 at this
	address and do nothing more.
---------------------------------------------------------------*/

WRITE_HANDLER( atarigen_slapstic59_w )
{
	update_bank(slapstic_tweak(offset>>1));
}


/*---------------------------------------------------------------
	atarigen_slapstic59_r: Tweaks the slapstic59 at the appropriate
	address and then reads a word from the underlying memory.
---------------------------------------------------------------*/

READ_HANDLER( atarigen_slapstic59_r )
{
	offset>>=1;

	/* fetch the result from the current bank first */
	int result = atarigen_slapstic59[offset & 0xfff];

	/* then determine the new one */
	update_bank(slapstic_tweak(offset));
	return result;
}




/*---------------------------------------------------------------
	atarigen_slapstic59_init: Installs memory handlers for the
	slapstic59 and sets the chip number.
---------------------------------------------------------------*/

void atarigen_slapstic59_init(int cpunum, int base, int chipnum)
{
	atarigen_slapstic59_num = chipnum;
	atarigen_slapstic59 = NULL;

	/* if we have a chip, install it */
	if (chipnum)
	{
		/* initialize the slapstic59 */
		slapstic_init(chipnum);

		/* install the memory handlers */
		atarigen_slapstic59 = install_mem_read_handler(cpunum, base, base + 0x7fff, atarigen_slapstic59_r);
		atarigen_slapstic59 = install_mem_write_handler(cpunum, base, base + 0x7fff, atarigen_slapstic59_w);

		/* allocate memory for a copy of bank 0 */
		memcpy(atarigen_slapstic59_bank0, atarigen_slapstic59, 0x2000);
	}
}


/*---------------------------------------------------------------
	atarigen_slapstic59_reset: Makes the selected slapstic59 number
	active and resets its state.
---------------------------------------------------------------*/

void atarigen_slapstic59_reset(void)
{
	if (atarigen_slapstic59_num)
	{
		slapstic_reset();
		update_bank(slapstic_bank());
	}
}

