#define VERBOSE 0


#include "driver.h"
#include "vidhrdw/konamiic.h"

static unsigned char *colorram,*videoram1,*videoram2;
static int layer;

/*
	This recursive function doesn't use additional memory
	(it could be easily converted into an iterative one).
	It's called shuffle because it mimics the shuffling of a deck of cards.
*/
static void shuffle(UINT16 *buf,int len)
{
	int i;
	UINT16 t;

	if (len == 2) return;

	if (len % 4) exit(1);   /* must not happen */

	len /= 2;

	for (i = 0;i < len/2;i++)
	{
		t = buf[(len>>1) + i];
		buf[(len>>1) + i] = buf[len + i];
		buf[len + i] = t;
	}

	shuffle(buf,len);
	shuffle(buf + len,len);
}


/* helper function to join two 16-bit ROMs and form a 32-bit data stream */
void konami_rom_deinterleave_2(int mem_region)
{
	shuffle((UINT16 *)memory_region(mem_region),memory_region_length(mem_region)/2);
}

/* helper function to join four 16-bit ROMs and form a 64-bit data stream */
void konami_rom_deinterleave_4(int mem_region)
{
	konami_rom_deinterleave_2(mem_region);
	konami_rom_deinterleave_2(mem_region);
}




static int K052109_memory_region;
static int K052109_gfxnum;
static void (*K052109_callback)(int tilemap,int bank,int *code,int *color);
static unsigned char *K052109_ram;
static unsigned char *K052109_videoram_F,*K052109_videoram2_F,*K052109_colorram_F;
static unsigned char *K052109_videoram_A,*K052109_videoram2_A,*K052109_colorram_A;
static unsigned char *K052109_videoram_B,*K052109_videoram2_B,*K052109_colorram_B;
static unsigned char K052109_charrombank[4];
static int has_extra_video_ram;
static int K052109_RMRD_line;
static int K052109_tileflip_enable;
static int K052109_irq_enabled;
static unsigned char K052109_romsubbank,K052109_scrollctrl;
static struct tilemap *K052109_tilemap[3];



/***************************************************************************

  Callbacks for the TileMap code

***************************************************************************/

/*
  data format:
  video RAM    xxxxxxxx  tile number (low 8 bits)
  color RAM    xxxx----  depends on external connections (usually color and banking)
  color RAM    ----xx--  bank select (0-3): these bits are replaced with the 2
                         bottom bits of the bank register before being placed on
                         the output pins. The other two bits of the bank register are
                         placed on the CAB1 and CAB2 output pins.
  color RAM    ------xx  depends on external connections (usually banking, flip)
*/

static void tilemap0_preupdate(void)
{
	colorram = K052109_colorram_F;
	videoram1 = K052109_videoram_F;
	videoram2 = K052109_videoram2_F;
	layer = 0;
}

static void tilemap1_preupdate(void)
{
	colorram = K052109_colorram_A;
	videoram1 = K052109_videoram_A;
	videoram2 = K052109_videoram2_A;
	layer = 1;
}

static void tilemap2_preupdate(void)
{
	colorram = K052109_colorram_B;
	videoram1 = K052109_videoram_B;
	videoram2 = K052109_videoram2_B;
	layer = 2;
}

static void K052109_get_tile_info(int tile_index)
{
	int flipy = 0;
	int code = videoram1[tile_index] + 256 * videoram2[tile_index];
	int color = colorram[tile_index];
	int bank = K052109_charrombank[(color & 0x0c) >> 2];
if (has_extra_video_ram) bank = (color & 0x0c) >> 2;	/* kludge for X-Men */
	color = (color & 0xf3) | ((bank & 0x03) << 2);
	bank >>= 2;

	flipy = color & 0x02;

	tile_info.flags = 0;

	(*K052109_callback)(layer,bank,&code,&color);

	SET_TILE_INFO(K052109_gfxnum,code,color);

	/* if the callback set flip X but it is not enabled, turn it off */
	if (!(K052109_tileflip_enable & 1)) tile_info.flags &= ~TILE_FLIPX;

	/* if flip Y is enabled and the attribute but is set, turn it on */
	if (flipy && (K052109_tileflip_enable & 2)) tile_info.flags |= TILE_FLIPY;
}



int K052109_vh_start(int gfx_memory_region,int plane0,int plane1,int plane2,int plane3,
		void (*callback)(int tilemap,int bank,int *code,int *color))
{
	int gfx_index;
	static struct GfxLayout charlayout =
	{
		8,8,
		0,				/* filled in later */
		4,
		{ 0, 0, 0, 0 },	/* filled in later */
		{ 0, 1, 2, 3, 4, 5, 6, 7 },
		{ 0*32, 1*32, 2*32, 3*32, 4*32, 5*32, 6*32, 7*32 },
		32*8
	};


	/* find first empty slot to decode gfx */
	for (gfx_index = 0; gfx_index < MAX_GFX_ELEMENTS; gfx_index++)
		if (Machine->gfx[gfx_index] == 0)
			break;
	if (gfx_index == MAX_GFX_ELEMENTS)
		return 1;

	/* tweak the structure for the number of tiles we have */
	charlayout.total = memory_region_length(gfx_memory_region) / 32;
	charlayout.planeoffset[0] = plane3 * 8;
	charlayout.planeoffset[1] = plane2 * 8;
	charlayout.planeoffset[2] = plane1 * 8;
	charlayout.planeoffset[3] = plane0 * 8;

	/* decode the graphics */
	Machine->gfx[gfx_index] = decodegfx(memory_region(gfx_memory_region),&charlayout);
	if (!Machine->gfx[gfx_index])
		return 1;

	/* set the color information */
	Machine->gfx[gfx_index]->colortable = Machine->remapped_colortable;
	Machine->gfx[gfx_index]->total_colors = Machine->drv->color_table_len / 16;

	K052109_memory_region = gfx_memory_region;
	K052109_gfxnum = gfx_index;
	K052109_callback = callback;
	K052109_RMRD_line = CLEAR_LINE;

	has_extra_video_ram = 0;

	K052109_tilemap[0] = tilemap_create(K052109_get_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,32);
	K052109_tilemap[1] = tilemap_create(K052109_get_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,32);
	K052109_tilemap[2] = tilemap_create(K052109_get_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,32);

	K052109_ram = (unsigned char*)malloc(0x6000);

	if (!K052109_ram || !K052109_tilemap[0] || !K052109_tilemap[1] || !K052109_tilemap[2])
	{
		K052109_vh_stop();
		return 1;
	}
	memset(K052109_ram,0,0x6000);

	K052109_colorram_F = &K052109_ram[0x0000];
	K052109_colorram_A = &K052109_ram[0x0800];
	K052109_colorram_B = &K052109_ram[0x1000];
	K052109_videoram_F = &K052109_ram[0x2000];
	K052109_videoram_A = &K052109_ram[0x2800];
	K052109_videoram_B = &K052109_ram[0x3000];
	K052109_videoram2_F = &K052109_ram[0x4000];
	K052109_videoram2_A = &K052109_ram[0x4800];
	K052109_videoram2_B = &K052109_ram[0x5000];

	K052109_tilemap[0]->transparent_pen = 0;
	K052109_tilemap[1]->transparent_pen = 0;
	K052109_tilemap[2]->transparent_pen = 0;

	return 0;
}

void K052109_vh_stop(void)
{
	free(K052109_ram);
	K052109_ram = 0;
}



READ_HANDLER( K052109_r )
{
	if (K052109_RMRD_line == CLEAR_LINE)
	{
//		if ((offset & 0x1fff) >= 0x1800)
//		{
//			if (offset >= 0x180c && offset < 0x1834)
//			{	/* A y scroll */	}
//			else if (offset >= 0x1a00 && offset < 0x1c00)
//			{	/* A x scroll */	}
//			else if (offset == 0x1d00)
//			{	/* read for bitwise operations before writing */	}
//			else if (offset >= 0x380c && offset < 0x3834)
//			{	/* B y scroll */	}
//			else if (offset >= 0x3a00 && offset < 0x3c00)
//			{	/* B x scroll */	}
//			else
//logerror("%04x: read from unknown 052109 address %04x\n",cpu_get_pc(),offset);
//		}

		return K052109_ram[offset];
	}
	else	/* Punk Shot and TMNT read from 0000-1fff, Aliens from 2000-3fff */
	{
		int code = (offset & 0x1fff) >> 5;
		int color = K052109_romsubbank;
		int bank = K052109_charrombank[(color & 0x0c) >> 2] >> 2;   /* discard low bits (TMNT) */
		int addr;

if (has_extra_video_ram) code |= color << 8;	/* kludge for X-Men */
else
		(*K052109_callback)(0,bank,&code,&color);

		addr = (code << 5) + (offset & 0x1f);
		addr &= memory_region_length(K052109_memory_region)-1;

#if 0
	usrintf_showmessage("%04x: off%04x sub%02x (bnk%x) adr%06x",cpu_get_pc(),offset,K052109_romsubbank,bank,addr);
#endif

		return memory_region(K052109_memory_region)[addr];
	}
}

WRITE_HANDLER( K052109_w )
{
	if ((offset & 0x1fff) < 0x1800) /* tilemap RAM */
	{
		if (K052109_ram[offset] != data)
		{
			if (offset >= 0x4000) has_extra_video_ram = 1;  /* kludge for X-Men */
			K052109_ram[offset] = data;
			tilemap_mark_tile_dirty(K052109_tilemap[(offset & 0x1800) >> 11],offset & 0x7ff);
		}
	}
	else	/* control registers */
	{
		K052109_ram[offset] = data;

		if (offset >= 0x180c && offset < 0x1834)
		{	/* A y scroll */	}
		else if (offset >= 0x1a00 && offset < 0x1c00)
		{	/* A x scroll */	}
		else if (offset == 0x1c80)
		{
if (K052109_scrollctrl != data)
{
#if 0
usrintf_showmessage("scrollcontrol = %02x",data);
#endif
//logerror("%04x: rowscrollcontrol = %02x\n",cpu_get_pc(),data);
			K052109_scrollctrl = data;
}
		}
		else if (offset == 0x1d00)
		{
#if VERBOSE
logerror("%04x: 052109 register 1d00 = %02x\n",cpu_get_pc(),data);
#endif
			/* bit 2 = irq enable */
			/* the custom chip can also generate NMI and FIRQ, for use with a 6809 */
			K052109_irq_enabled = data & 0x04;
		}
		else if (offset == 0x1d80)
		{
			int dirty = 0;

			if (K052109_charrombank[0] != (data & 0x0f)) dirty |= 1;
			if (K052109_charrombank[1] != ((data >> 4) & 0x0f)) dirty |= 2;
			if (dirty)
			{
				int i;

				K052109_charrombank[0] = data & 0x0f;
				K052109_charrombank[1] = (data >> 4) & 0x0f;

				for (i = 0;i < 0x1800;i++)
				{
					int bank = (K052109_ram[i]&0x0c) >> 2;
					if ((bank == 0 && (dirty & 1)) || (bank == 1 && dirty & 2))
					{
						tilemap_mark_tile_dirty(K052109_tilemap[(i & 0x1800) >> 11],i & 0x7ff);
					}
				}
			}
		}
		else if (offset == 0x1e00)
		{
//logerror("%04x: 052109 register 1e00 = %02x\n",cpu_get_pc(),data);
			K052109_romsubbank = data;
		}
		else if (offset == 0x1e80)
		{
//if ((data & 0xfe)) logerror("%04x: 052109 register 1e80 = %02x\n",cpu_get_pc(),data);
			tilemap_set_flip(K052109_tilemap[0],(data & 1) ? (TILEMAP_FLIPY | TILEMAP_FLIPX) : 0);
			tilemap_set_flip(K052109_tilemap[1],(data & 1) ? (TILEMAP_FLIPY | TILEMAP_FLIPX) : 0);
			tilemap_set_flip(K052109_tilemap[2],(data & 1) ? (TILEMAP_FLIPY | TILEMAP_FLIPX) : 0);
			if (K052109_tileflip_enable != ((data & 0x06) >> 1))
			{
				K052109_tileflip_enable = ((data & 0x06) >> 1);

				tilemap_mark_all_tiles_dirty(K052109_tilemap[0]);
				tilemap_mark_all_tiles_dirty(K052109_tilemap[1]);
				tilemap_mark_all_tiles_dirty(K052109_tilemap[2]);
			}
		}
		else if (offset == 0x1f00)
		{
			int dirty = 0;

			if (K052109_charrombank[2] != (data & 0x0f)) dirty |= 1;
			if (K052109_charrombank[3] != ((data >> 4) & 0x0f)) dirty |= 2;
			if (dirty)
			{
				int i;

				K052109_charrombank[2] = data & 0x0f;
				K052109_charrombank[3] = (data >> 4) & 0x0f;

				for (i = 0;i < 0x1800;i++)
				{
					int bank = (K052109_ram[i] & 0x0c) >> 2;
					if ((bank == 2 && (dirty & 1)) || (bank == 3 && dirty & 2))
						tilemap_mark_tile_dirty(K052109_tilemap[(i & 0x1800) >> 11],i & 0x7ff);
				}
			}
		}
		else if (offset >= 0x380c && offset < 0x3834)
		{	/* B y scroll */	}
		else if (offset >= 0x3a00 && offset < 0x3c00)
		{	/* B x scroll */	}
//		else
//logerror("%04x: write %02x to unknown 052109 address %04x\n",cpu_get_pc(),data,offset);
	}
}

void K052109_set_RMRD_line(int state)
{
	K052109_RMRD_line = state;
}


void K052109_tilemap_update(void)
{
#if 0
{
usrintf_showmessage("%x %x %x %x",
	K052109_charrombank[0],
	K052109_charrombank[1],
	K052109_charrombank[2],
	K052109_charrombank[3]);
}
#endif
	if ((K052109_scrollctrl & 0x03) == 0x02)
	{
		int xscroll,yscroll,offs;
		unsigned char *scrollram = &K052109_ram[0x1a00];


		tilemap_set_scroll_rows(K052109_tilemap[1],256);
		tilemap_set_scroll_cols(K052109_tilemap[1],1);
		yscroll = K052109_ram[0x180c];
		tilemap_set_scrolly(K052109_tilemap[1],0,yscroll);
		for (offs = 0;offs < 256;offs++)
		{
			xscroll = scrollram[2*(offs&0xfff8)+0] + 256 * scrollram[2*(offs&0xfff8)+1];
			xscroll -= 6;
			tilemap_set_scrollx(K052109_tilemap[1],(offs+yscroll)&0xff,xscroll);
		}
	}
	else if ((K052109_scrollctrl & 0x03) == 0x03)
	{
		int xscroll,yscroll,offs;
		unsigned char *scrollram = &K052109_ram[0x1a00];


		tilemap_set_scroll_rows(K052109_tilemap[1],256);
		tilemap_set_scroll_cols(K052109_tilemap[1],1);
		yscroll = K052109_ram[0x180c];
		tilemap_set_scrolly(K052109_tilemap[1],0,yscroll);
		for (offs = 0;offs < 256;offs++)
		{
			xscroll = scrollram[2*offs+0] + 256 * scrollram[2*offs+1];
			xscroll -= 6;
			tilemap_set_scrollx(K052109_tilemap[1],(offs+yscroll)&0xff,xscroll);
		}
	}
	else if ((K052109_scrollctrl & 0x04) == 0x04)
	{
		int xscroll,yscroll,offs;
		unsigned char *scrollram = &K052109_ram[0x1800];


		tilemap_set_scroll_rows(K052109_tilemap[1],1);
		tilemap_set_scroll_cols(K052109_tilemap[1],512);
		xscroll = K052109_ram[0x1a00] + 256 * K052109_ram[0x1a01];
		xscroll -= 6;
		tilemap_set_scrollx(K052109_tilemap[1],0,xscroll);
		for (offs = 0;offs < 512;offs++)
		{
			yscroll = scrollram[offs/8];
			tilemap_set_scrolly(K052109_tilemap[1],(offs+xscroll)&0x1ff,yscroll);
		}
	}
	else
	{
		int xscroll,yscroll;
		unsigned char *scrollram = &K052109_ram[0x1a00];


		tilemap_set_scroll_rows(K052109_tilemap[1],1);
		tilemap_set_scroll_cols(K052109_tilemap[1],1);
		xscroll = scrollram[0] + 256 * scrollram[1];
		xscroll -= 6;
		yscroll = K052109_ram[0x180c];
		tilemap_set_scrollx(K052109_tilemap[1],0,xscroll);
		tilemap_set_scrolly(K052109_tilemap[1],0,yscroll);
	}

	if ((K052109_scrollctrl & 0x18) == 0x10)
	{
		int xscroll,yscroll,offs;
		unsigned char *scrollram = &K052109_ram[0x3a00];


		tilemap_set_scroll_rows(K052109_tilemap[2],256);
		tilemap_set_scroll_cols(K052109_tilemap[2],1);
		yscroll = K052109_ram[0x380c];
		tilemap_set_scrolly(K052109_tilemap[2],0,yscroll);
		for (offs = 0;offs < 256;offs++)
		{
			xscroll = scrollram[2*(offs&0xfff8)+0] + 256 * scrollram[2*(offs&0xfff8)+1];
			xscroll -= 6;
			tilemap_set_scrollx(K052109_tilemap[2],(offs+yscroll)&0xff,xscroll);
		}
	}
	else if ((K052109_scrollctrl & 0x18) == 0x18)
	{
		int xscroll,yscroll,offs;
		unsigned char *scrollram = &K052109_ram[0x3a00];


		tilemap_set_scroll_rows(K052109_tilemap[2],256);
		tilemap_set_scroll_cols(K052109_tilemap[2],1);
		yscroll = K052109_ram[0x380c];
		tilemap_set_scrolly(K052109_tilemap[2],0,yscroll);
		for (offs = 0;offs < 256;offs++)
		{
			xscroll = scrollram[2*offs+0] + 256 * scrollram[2*offs+1];
			xscroll -= 6;
			tilemap_set_scrollx(K052109_tilemap[2],(offs+yscroll)&0xff,xscroll);
		}
	}
	else if ((K052109_scrollctrl & 0x20) == 0x20)
	{
		int xscroll,yscroll,offs;
		unsigned char *scrollram = &K052109_ram[0x3800];


		tilemap_set_scroll_rows(K052109_tilemap[2],1);
		tilemap_set_scroll_cols(K052109_tilemap[2],512);
		xscroll = K052109_ram[0x3a00] + 256 * K052109_ram[0x3a01];
		xscroll -= 6;
		tilemap_set_scrollx(K052109_tilemap[2],0,xscroll);
		for (offs = 0;offs < 512;offs++)
		{
			yscroll = scrollram[offs/8];
			tilemap_set_scrolly(K052109_tilemap[2],(offs+xscroll)&0x1ff,yscroll);
		}
	}
	else
	{
		int xscroll,yscroll;
		unsigned char *scrollram = &K052109_ram[0x3a00];


		tilemap_set_scroll_rows(K052109_tilemap[2],1);
		tilemap_set_scroll_cols(K052109_tilemap[2],1);
		xscroll = scrollram[0] + 256 * scrollram[1];
		xscroll -= 6;
		yscroll = K052109_ram[0x380c];
		tilemap_set_scrollx(K052109_tilemap[2],0,xscroll);
		tilemap_set_scrolly(K052109_tilemap[2],0,yscroll);
	}

	tilemap0_preupdate(); tilemap_update(K052109_tilemap[0]);
	tilemap1_preupdate(); tilemap_update(K052109_tilemap[1]);
	tilemap2_preupdate(); tilemap_update(K052109_tilemap[2]);
}

void K052109_tilemap_draw(struct osd_bitmap *bitmap,int num,int flags)
{
	tilemap_draw(bitmap,K052109_tilemap[num],flags);
}

int K052109_is_IRQ_enabled(void)
{
	return K052109_irq_enabled;
}



static int K053247_memory_region;
static struct GfxElement *K053247_gfx;
static void (*K053247_callback)(int *code,int *color,int *priority);
static int K053246_OBJCHA_line;
static int K053246_romoffset;
static int K053247_flipscreenX,K053247_flipscreenY;
static int K053247_spriteoffsX,K053247_spriteoffsY;
static unsigned char *K053247_ram;
static int K053247_irq_enabled;


int K053247_vh_start(int gfx_memory_region,int plane0,int plane1,int plane2,int plane3,
		void (*callback)(int *code,int *color,int *priority))
{
	int gfx_index;
	static struct GfxLayout spritelayout =
	{
		16,16,
		0,				/* filled in later */
		4,
		{ 0, 0, 0, 0 },	/* filled in later */
		{ 2*4, 3*4, 0*4, 1*4, 6*4, 7*4, 4*4, 5*4,
				10*4, 11*4, 8*4, 9*4, 14*4, 15*4, 12*4, 13*4 },
		{ 0*64, 1*64, 2*64, 3*64, 4*64, 5*64, 6*64, 7*64,
				8*64, 9*64, 10*64, 11*64, 12*64, 13*64, 14*64, 15*64 },
		128*8
	};


  /* find first empty slot to decode gfx */
	for (gfx_index = 0; gfx_index < MAX_GFX_ELEMENTS; gfx_index++)
		if (Machine->gfx[gfx_index] == 0)
			break;
	if (gfx_index == MAX_GFX_ELEMENTS)
		return 1;

	/* tweak the structure for the number of tiles we have */
	spritelayout.total = memory_region_length(gfx_memory_region) / 128;
	spritelayout.planeoffset[0] = plane0;
	spritelayout.planeoffset[1] = plane1;
	spritelayout.planeoffset[2] = plane2;
	spritelayout.planeoffset[3] = plane3;

	/* decode the graphics */
	Machine->gfx[gfx_index] = decodegfx(memory_region(gfx_memory_region),&spritelayout);
	if (!Machine->gfx[gfx_index])
		return 1;

	/* set the color information */
	Machine->gfx[gfx_index]->colortable = Machine->remapped_colortable;
	Machine->gfx[gfx_index]->total_colors = Machine->drv->color_table_len / 16;

	K053247_memory_region = gfx_memory_region;
	K053247_gfx = Machine->gfx[gfx_index];
	K053247_callback = callback;
	K053246_OBJCHA_line = CLEAR_LINE;
  
	K053247_ram = (unsigned char*)malloc(0x1000);
	if (!K053247_ram) return 1;

  memset(K053247_ram,0,0x1000);

	return 0;
}

void K053247_vh_stop(void)
{
	free(K053247_ram);
	K053247_ram = 0;
}

READ_HANDLER( K053247_word_r )
{
	return READ_WORD(&K053247_ram[offset]);
}

WRITE_HANDLER( K053247_word_w )
{
	COMBINE_WORD_MEM(&K053247_ram[offset],data);
}

READ_HANDLER( K053247_r )
{
	int shift = ((offset & 1) ^ 1) << 3;
	return (READ_WORD(&K053247_ram[offset & ~1]) >> shift) & 0xff;
}

WRITE_HANDLER( K053247_w )
{
	int shift = ((offset & 1) ^ 1) << 3;
	offset &= ~1;
	COMBINE_WORD_MEM(&K053247_ram[offset],(0xff000000 >> shift) | ((data & 0xff) << shift));
}

READ_HANDLER( K053246_r )
{
	if (K053246_OBJCHA_line == ASSERT_LINE)
	{
		int addr;


		addr = 2 * K053246_romoffset + ((offset & 1) ^ 1);
		addr &= memory_region_length(K053247_memory_region)-1;

#if 0
	usrintf_showmessage("%04x: offset %02x addr %06x",cpu_get_pc(),offset,addr);
#endif

		return memory_region(K053247_memory_region)[addr];
	}
	else
	{
//logerror("%04x: read from unknown 053244 address %x\n",cpu_get_pc(),offset);
		return 0;
	}
}

WRITE_HANDLER( K053246_w )
{
	if (offset == 0x00)
		K053247_spriteoffsX = (K053247_spriteoffsX & 0x00ff) | (data << 8);
	else if (offset == 0x01)
		K053247_spriteoffsX = (K053247_spriteoffsX & 0xff00) | data;
	else if (offset == 0x02)
		K053247_spriteoffsY = (K053247_spriteoffsY & 0x00ff) | (data << 8);
	else if (offset == 0x03)
		K053247_spriteoffsY = (K053247_spriteoffsY & 0xff00) | data;
	else if (offset == 0x05)
	{
		/* bit 0/1 = flip screen */
		K053247_flipscreenX = data & 0x01;
		K053247_flipscreenY = data & 0x02;

		/* bit 2 = unknown */

		/* bit 4 = interrupt enable */
		K053247_irq_enabled = data & 0x10;

		/* bit 5 = unknown */

//logerror("%04x: write %02x to 053246 address 5\n",cpu_get_pc(),data);
	}
	else if (offset >= 0x04 && offset < 0x08)   /* only 4,6,7 - 5 is handled above */
	{
		offset = 8*(((offset & 0x03) ^ 0x01) - 1);
		K053246_romoffset = (K053246_romoffset & ~(0xff << offset)) | (data << offset);
		return;
	}
//	else
//logerror("%04x: write %02x to unknown 053246 address %x\n",cpu_get_pc(),data,offset);
}

READ_HANDLER( K053246_word_r )
{
	return K053246_r(offset + 1) | (K053246_r(offset) << 8);
}

WRITE_HANDLER( K053246_word_w )
{
	if ((data & 0xff000000) == 0)
		K053246_w(offset,(data >> 8) & 0xff);
	if ((data & 0x00ff0000) == 0)
		K053246_w(offset + 1,data & 0xff);
}

void K053246_set_OBJCHA_line(int state)
{
	K053246_OBJCHA_line = state;
}

/*
 * Sprite Format
 * ------------------
 *
 * Word | Bit(s)           | Use
 * -----+-fedcba9876543210-+----------------
 *   0  | x--------------- | active (show this sprite)
 *   0  | -x-------------- | maintain aspect ratio (when set, zoom y acts on both axis)
 *   0  | --x------------- | flip y
 *   0  | ---x------------ | flip x
 *   0  | ----xxxx-------- | sprite size (see below)
 *   0  | ---------xxxxxxx | priority order
 *   1  | xxxxxxxxxxxxxxxx | sprite code
 *   2  | ------xxxxxxxxxx | y position
 *   3  | ------xxxxxxxxxx | x position
 *   4  | xxxxxxxxxxxxxxxx | zoom y (0x40 = normal, <0x40 = enlarge, >0x40 = reduce)
 *   5  | xxxxxxxxxxxxxxxx | zoom x (0x40 = normal, <0x40 = enlarge, >0x40 = reduce)
 *   6  | x--------------- | mirror y (top half is drawn as mirror image of the bottom)
 *   6  | -x-------------- | mirror x (right half is drawn as mirror image of the left)
 *   6  | -----x---------- | shadow
 *   6  | xxxxxxxxxxxxxxxx | "color", but depends on external connections
 *   7  | ---------------- |
 *
 * shadow enables transparent shadows. Note that it applies to pen 0x0f ONLY.
 * The rest of the sprite remains normal.
 */

void K053247_sprites_draw(struct osd_bitmap *bitmap)
{
#define NUM_SPRITES 256
	int offs;
	int pri_code;
	int sortedlist[NUM_SPRITES];

	for (offs = 0;offs < NUM_SPRITES;offs++)
		sortedlist[offs] = -1;

	/* prebuild a sorted table */
	for (offs = 0;offs < 0x1000;offs += 16)
	{
//		if (READ_WORD(&K053247_ram[offs]) & 0x8000)
		sortedlist[READ_WORD(&K053247_ram[offs]) & 0x00ff] = offs;
	}

	for (pri_code = 0;pri_code < NUM_SPRITES;pri_code++)
	{
		int ox,oy,color,code,size,w,h,x,y,xa,ya,flipx,flipy,mirrorx,mirrory,zoomx,zoomy,pri;
		/* sprites can be grouped up to 8x8. The draw order is
			 0  1  4  5 16 17 20 21
			 2  3  6  7 18 19 22 23
			 8  9 12 13 24 25 28 29
			10 11 14 15 26 27 30 31
			32 33 36 37 48 49 52 53
			34 35 38 39 50 51 54 55
			40 41 44 45 56 57 60 61
			42 43 46 47 58 59 62 63
		*/
		static int xoffset[8] = { 0, 1, 4, 5, 16, 17, 20, 21 };
		static int yoffset[8] = { 0, 2, 8, 10, 32, 34, 40, 42 };
		static int offsetkludge;


		offs = sortedlist[pri_code];
		if (offs == -1) continue;

		if ((READ_WORD(&K053247_ram[offs]) & 0x8000) == 0) continue;

		code = READ_WORD(&K053247_ram[offs+0x02]);
		color = READ_WORD(&K053247_ram[offs+0x0c]);
		pri = 0;

		(*K053247_callback)(&code,&color,&pri);

		size = (READ_WORD(&K053247_ram[offs]) & 0x0f00) >> 8;

		w = 1 << (size & 0x03);
		h = 1 << ((size >> 2) & 0x03);

		/* the sprite can start at any point in the 8x8 grid. We have to */
		/* adjust the offsets to draw it correctly. Simpsons does this all the time. */
		xa = 0;
		ya = 0;
		if (code & 0x01) xa += 1;
		if (code & 0x02) ya += 1;
		if (code & 0x04) xa += 2;
		if (code & 0x08) ya += 2;
		if (code & 0x10) xa += 4;
		if (code & 0x20) ya += 4;
		code &= ~0x3f;


		/* zoom control:
		   0x40 = normal scale
		  <0x40 enlarge (0x20 = double size)
		  >0x40 reduce (0x80 = half size)
		*/
		zoomy = READ_WORD(&K053247_ram[offs+0x08]);
		if (zoomy > 0x2000) continue;
		if (zoomy) zoomy = (0x400000+zoomy/2) / zoomy;
		else zoomy = 2 * 0x400000;
		if ((READ_WORD(&K053247_ram[offs]) & 0x4000) == 0)
		{
			zoomx = READ_WORD(&K053247_ram[offs+0x0a]);
			if (zoomx > 0x2000) continue;
			if (zoomx) zoomx = (0x400000+zoomx/2) / zoomx;
			else zoomx = 2 * 0x400000;
		}
		else zoomx = zoomy;

		ox = READ_WORD(&K053247_ram[offs+0x06]);
		oy = READ_WORD(&K053247_ram[offs+0x04]);

/* TODO: it is not known how the global Y offset works */
switch (K053247_spriteoffsY)
{
	case 0x0261:	/* simpsons */
	case 0x0262:	/* simpsons (dreamland) */
	case 0x0263:	/* simpsons (dreamland) */
	case 0x0264:	/* simpsons (dreamland) */
	case 0x0265:	/* simpsons (dreamland) */
	case 0x006d:	/* simpsons flip (dreamland) */
	case 0x006e:	/* simpsons flip (dreamland) */
	case 0x006f:	/* simpsons flip (dreamland) */
	case 0x0070:	/* simpsons flip (dreamland) */
	case 0x0071:	/* simpsons flip */
		offsetkludge = 0x017;
		break;
	case 0x02f7:	/* vendetta (level 4 boss) */
	case 0x02f8:	/* vendetta (level 4 boss) */
	case 0x02f9:	/* vendetta (level 4 boss) */
	case 0x02fa:	/* vendetta */
	case 0x02fb:	/* vendetta (fat guy jumping) */
	case 0x02fc:	/* vendetta (fat guy jumping) */
	case 0x02fd:	/* vendetta (fat guy jumping) */
	case 0x02fe:	/* vendetta (fat guy jumping) */
	case 0x02ff:	/* vendetta (fat guy jumping) */
	case 0x03f7:	/* vendetta flip (level 4 boss) */
	case 0x03f8:	/* vendetta flip (level 4 boss) */
	case 0x03f9:	/* vendetta flip (level 4 boss) */
	case 0x03fa:	/* vendetta flip */
	case 0x03fb:	/* vendetta flip (fat guy jumping) */
	case 0x03fc:	/* vendetta flip (fat guy jumping) */
	case 0x03fd:	/* vendetta flip (fat guy jumping) */
	case 0x03fe:	/* vendetta flip (fat guy jumping) */
	case 0x03ff:	/* vendetta flip (fat guy jumping) */
		offsetkludge = 0x006;
		break;
	case 0x0292:	/* xmen */
	case 0x0072:	/* xmen flip */
		offsetkludge = -0x002;
		break;
	default:
		offsetkludge = 0;
			usrintf_showmessage("unknown spriteoffsY %04x",K053247_spriteoffsY);
		break;
}

		flipx = READ_WORD(&K053247_ram[offs]) & 0x1000;
		flipy = READ_WORD(&K053247_ram[offs]) & 0x2000;
		mirrorx = READ_WORD(&K053247_ram[offs+0x0c]) & 0x4000;
		mirrory = READ_WORD(&K053247_ram[offs+0x0c]) & 0x8000;

		if (K053247_flipscreenX)
		{
			ox = -ox;
			if (!mirrorx) flipx = !flipx;
		}
		if (K053247_flipscreenY)
		{
			oy = -oy;
			if (!mirrory) flipy = !flipy;
		}

		ox = (ox + 0x35 - K053247_spriteoffsX) & 0x3ff;
		if (ox >= 768) ox -= 1024;
		oy = (-(oy + K053247_spriteoffsY + offsetkludge)) & 0x3ff;
		if (oy >= 640) oy -= 1024;

		/* the coordinates given are for the *center* of the sprite */
		ox -= (zoomx * w) >> 13;
		oy -= (zoomy * h) >> 13;

		for (y = 0;y < h;y++)
		{
			int sx,sy,zw,zh;

			sy = oy + ((zoomy * y + (1<<11)) >> 12);
			zh = (oy + ((zoomy * (y+1) + (1<<11)) >> 12)) - sy;

			for (x = 0;x < w;x++)
			{
				int c,fx,fy;

				sx = ox + ((zoomx * x + (1<<11)) >> 12);
				zw = (ox + ((zoomx * (x+1) + (1<<11)) >> 12)) - sx;
				c = code;
				if (mirrorx)
				{
					if ((flipx == 0) ^ (2*x < w))
					{
						/* mirror left/right */
						c += xoffset[(w-1-x+xa)&7];
						fx = 1;
					}
					else
					{
						c += xoffset[(x+xa)&7];
						fx = 0;
					}
				}
				else
				{
					if (flipx) c += xoffset[(w-1-x+xa)&7];
					else c += xoffset[(x+xa)&7];
					fx = flipx;
				}
				if (mirrory)
				{
					if ((flipy == 0) ^ (2*y >= h))
					{
						/* mirror top/bottom */
						c += yoffset[(h-1-y+ya)&7];
						fy = 1;
					}
					else
					{
						c += yoffset[(y+ya)&7];
						fy = 0;
					}
				}
				else
				{
					if (flipy) c += yoffset[(h-1-y+ya)&7];
					else c += yoffset[(y+ya)&7];
					fy = flipy;
				}

				if (zoomx == 0x10000 && zoomy == 0x10000)
				{
					/* hack to simulate shadow */
					if (READ_WORD(&K053247_ram[offs+0x0c]) & 0x0400)
					{
						int o = K053247_gfx->colortable[16*color+15];
						K053247_gfx->colortable[16*color+15] = palette_transparent_pen;
						pdrawgfx(bitmap,K053247_gfx,
								c,
								color,
								fx,fy,
								sx,sy,
								&Machine->visible_area,TRANSPARENCY_PENS,(cpu_getcurrentframe() & 1) ? 0x8001 : 0x0001,pri);
						K053247_gfx->colortable[16*color+15] = o;
					}
					else
					{
						pdrawgfx(bitmap,K053247_gfx,
								c,
								color,
								fx,fy,
								sx,sy,
								&Machine->visible_area,TRANSPARENCY_PEN,0,pri);
					}
				}
				else
				{
					/* hack to simulate shadow */
					if (READ_WORD(&K053247_ram[offs+0x0c]) & 0x0400)
					{
						int o = K053247_gfx->colortable[16*color+15];
						K053247_gfx->colortable[16*color+15] = palette_transparent_pen;
						pdrawgfxzoom(bitmap,K053247_gfx,
								c,
								color,
								fx,fy,
								sx,sy,
								&Machine->visible_area,TRANSPARENCY_PENS,(cpu_getcurrentframe() & 1) ? 0x8001 : 0x0001,
								(zw << 16) / 16,(zh << 16) / 16,pri);
						K053247_gfx->colortable[16*color+15] = o;
					}
					else
					{
						pdrawgfxzoom(bitmap,K053247_gfx,
								c,
								color,
								fx,fy,
								sx,sy,
								&Machine->visible_area,TRANSPARENCY_PEN,0,
								(zw << 16) / 16,(zh << 16) / 16,pri);
					}
				}

				if (mirrory && h == 1)  /* Simpsons shadows */
				{
					if (zoomx == 0x10000 && zoomy == 0x10000)
					{
						/* hack to simulate shadow */
						if (READ_WORD(&K053247_ram[offs+0x0c]) & 0x0400)
						{
							int o = K053247_gfx->colortable[16*color+15];
							K053247_gfx->colortable[16*color+15] = palette_transparent_pen;
							pdrawgfx(bitmap,K053247_gfx,
									c,
									color,
									fx,!fy,
									sx,sy,
									&Machine->visible_area,TRANSPARENCY_PENS,(cpu_getcurrentframe() & 1) ? 0x8001 : 0x0001,pri);
							K053247_gfx->colortable[16*color+15] = o;
						}
						else
						{
							pdrawgfx(bitmap,K053247_gfx,
									c,
									color,
									fx,!fy,
									sx,sy,
									&Machine->visible_area,TRANSPARENCY_PEN,0,pri);
						}
					}
					else
					{
						/* hack to simulate shadow */
						if (READ_WORD(&K053247_ram[offs+0x0c]) & 0x0400)
						{
							int o = K053247_gfx->colortable[16*color+15];
							K053247_gfx->colortable[16*color+15] = palette_transparent_pen;
							pdrawgfxzoom(bitmap,K053247_gfx,
									c,
									color,
									fx,!fy,
									sx,sy,
									&Machine->visible_area,TRANSPARENCY_PENS,(cpu_getcurrentframe() & 1) ? 0x8001 : 0x0001,
									(zw << 16) / 16,(zh << 16) / 16,pri);
							K053247_gfx->colortable[16*color+15] = o;
						}
						else
						{
							pdrawgfxzoom(bitmap,K053247_gfx,
									c,
									color,
									fx,!fy,
									sx,sy,
									&Machine->visible_area,TRANSPARENCY_PEN,0,
									(zw << 16) / 16,(zh << 16) / 16,pri);
						}
					}
				}
			}
		}
	}
#if 0
if (keyboard_pressed(KEYCODE_D))
{
	FILE *fp;
	fp=fopen("SPRITE.DMP", "w+b");
	if (fp)
	{
		fwrite(K053247_ram, 0x1000, 1, fp);
		usrintf_showmessage("saved");
		fclose(fp);
	}
}
#endif
#undef NUM_SPRITES
}

void K053247_mark_sprites_colors(void)
{
	int offs,i;

	unsigned short palette_map[512];

	memset (palette_map, 0, sizeof (palette_map));

	/* sprites */
	for (offs = 0x1000-16;offs >= 0;offs -= 16)
	{
		if (READ_WORD(&K053247_ram[offs]) & 0x8000)
		{
			int code,color,pri;

			code = READ_WORD(&K053247_ram[offs+0x02]);
			color = READ_WORD(&K053247_ram[offs+0x0c]);
			pri = 0;
			(*K053247_callback)(&code,&color,&pri);
			palette_map[color] |= 0xffff;
		}
	}

	/* now build the final table */
	for (i = 0; i < 512; i++)
	{
		int usage = palette_map[i], j;
		if (usage)
		{
			for (j = 1; j < 16; j++)
				if (usage & (1 << j))
					palette_used_colors[i * 16 + j] |= PALETTE_COLOR_VISIBLE;
		}
	}
}

int K053247_is_IRQ_enabled(void)
{
	return K053247_irq_enabled;
}

static unsigned char K053251_ram[16];
static int K053251_palette_index[5];

WRITE_HANDLER( K053251_w )
{
            data &= 0x3f;

            if (K053251_ram[offset] != data)
	    {
	    K053251_ram[offset] = data;
	    if (offset == 9)
	    {
	    /* palette base index */
	    K053251_palette_index[0] = 32 * ((data >> 0) & 0x03);
	    K053251_palette_index[1] = 32 * ((data >> 2) & 0x03);
	    K053251_palette_index[2] = 32 * ((data >> 4) & 0x03);
	    tilemap_mark_all_tiles_dirty(ALL_TILEMAPS);
	    }
	    else if (offset == 10)
	    {
	    /* palette base index */
	    K053251_palette_index[3] = 16 * ((data >> 0) & 0x07);
	    K053251_palette_index[4] = 16 * ((data >> 3) & 0x07);
	    tilemap_mark_all_tiles_dirty(ALL_TILEMAPS);
	    }
	    }
}


int K053251_get_palette_index(int ci)
{
        return K053251_palette_index[ci];
}

int K053251_get_priority(int ci)
{
        return K053251_ram[ci];
}


static unsigned char K054000_ram[0x20];

WRITE_HANDLER( K054000_w )
{
	K054000_ram[offset] = data;
}

READ_HANDLER( K054000_r )
{
    int Acx,Acy,Aax,Aay;
    int Bcx,Bcy,Bax,Bay;

    if (offset != 0x18) return 0;

    Acx = (K054000_ram[0x01] << 16) | (K054000_ram[0x02] << 8) | K054000_ram[0x03];
    Acy = (K054000_ram[0x09] << 16) | (K054000_ram[0x0a] << 8) | K054000_ram[0x0b];
    /* TODO: this is a hack to make thndrx2 pass the startup check. It is certainly wrong. */
    if (K054000_ram[0x04] == 0xff) Acx+=3;
    if (K054000_ram[0x0c] == 0xff) Acy+=3;
    Aax = K054000_ram[0x06] + 1;
    Aay = K054000_ram[0x07] + 1;

    Bcx = (K054000_ram[0x15] << 16) | (K054000_ram[0x16] << 8) | K054000_ram[0x17];
    Bcy = (K054000_ram[0x11] << 16) | (K054000_ram[0x12] << 8) | K054000_ram[0x13];
    Bax = K054000_ram[0x0e] + 1;
    Bay = K054000_ram[0x0f] + 1;

    if (Acx + Aax < Bcx - Bax)
	return 1;

    if (Bcx + Bax < Acx - Aax)
	return 1;

    if (Acy + Aay < Bcy - Bay)
	return 1;

    if (Bcy + Bay < Acy - Aay)
	return 1;

    return 0;
}


