/*****************************************************************************

	Irem M90 system.  There is 1 video chip - NANAO GA-25, it produces
	2 tilemaps and sprites.  16 control bytes:

	0:  Playfield 1 X scroll
	2:  Playfield 1 Y scroll
	4:  Playfield 2 X scroll
	6:  Playfield 2 Y scroll
	8:  Bit 0x01 - unknown (set by hasamu)
	10: Playfield 1 control
		Bits0x03 - Playfield 1 VRAM base
 		Bit 0x04 - Playfield 1 width (0 is 64 tiles, 0x4 is 128 tiles)
		Bit 0x10 - Playfield 1 disable
		Bit 0x20 - Playfield 1 rowscroll enable
	12: Playfield 2 control
		Bits0x03 - Playfield 2 VRAM base
 		Bit 0x04 - Playfield 2 width (0 is 64 tiles, 0x4 is 128 tiles)
		Bit 0x10 - Playfield 2 disable
		Bit 0x20 - Playfield 2 rowscroll enable
	14: Bit 0x02 - unknown (set by hasamu)

	Emulation by Bryan McPhail, mish@tendril.co.uk, thanks to Chris Hardy!

*****************************************************************************/

#include "options.h"
#include "driver.h"
#include "vidhrdw/generic.h"

static unsigned char *m90_spriteram;
unsigned char *m90_video_data;
static struct tilemap *pf1_layer,*pf2_layer;
static int pf1_size = 0, pf2_size = 0;
static int m90_video_control_data[16];

static void get_tile_info(int tile_index,int layer,int page_mask)
{
	int tile,color;
	tile_index = (tile_index<<2) + ((m90_video_control_data[0xa+(layer<<1)] & page_mask)*0x4000);

	tile=m90_video_data[tile_index]+(m90_video_data[tile_index+1]<<8);
	color=m90_video_data[tile_index+2];
	SET_TILE_INFO( 0, tile, color&0xf )
	tile_info.flags = TILE_FLIPYX((color & 0xc0) >> 6);
	tile_info.priority = (color & 0x30) ? 1 : 0;
}


static void get_pf1_tile_info (int tile_index) { get_tile_info(tile_index,0,3); }
static void get_pf1w_tile_info(int tile_index) { get_tile_info(tile_index,0,2); }
static void get_pf2_tile_info (int tile_index) { get_tile_info(tile_index,1,3); }
static void get_pf2w_tile_info(int tile_index) { get_tile_info(tile_index,1,2); }


int m90_vh_start (void )
{
	free_disposable_regions(); // Added by AS - Make room for big tilemaps
	// Also had to remove having a regular and wide version tilemap of each layer
	// They don't actually change between each often so when it does change
	// it just recreates the tilemap.
	// It's probably faster this way as it's not maintaining 4 tilemaps

	pf1_size = 0, pf2_size = 1;
	pf1_layer =      tilemap_create(get_pf1_tile_info, tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,64);
	//pf1_wide_layer = tilemap_create(get_pf1w_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,128,64);
	//pf2_layer =      tilemap_create(get_pf2_tile_info, tilemap_scan_rows,TILEMAP_OPAQUE,8,8,64,64);
	//pf2_wide_layer = tilemap_create(get_pf2w_tile_info,tilemap_scan_rows,TILEMAP_OPAQUE,8,8,128,64);
	pf2_layer = tilemap_create(get_pf2w_tile_info,tilemap_scan_rows,TILEMAP_OPAQUE,8,8,128,64);

	if (!pf1_layer || !pf2_layer )
		return 1;

	tilemap_set_transparent_pen(pf1_layer,0);
	//tilemap_set_transparent_pen(pf1_wide_layer,0);

	return 0;
}

void m90_vh_stop ( void )
{
}

#if 0

static void m90_drawsprites(struct osd_bitmap *bitmap,const struct rectangle *cliprect)
{
	int offs;

	for (offs = 0x1f2;offs >= 0;offs -= 6)
	{
	    int x,y,sprite,colour,fx,fy,y_multi,i;

	    sprite = (m90_spriteram[offs+2] | (m90_spriteram[offs+3]<<8));
	    colour = (m90_spriteram[offs+1] >> 1) & 0x0f;

	    y = m90_spriteram[offs+0] | ((m90_spriteram[offs+1] & 0x01) << 8);
	    x = m90_spriteram[offs+4] | ((m90_spriteram[offs+5] & 0x01) << 8);

	    x = x - 16;
	    y = 512 - y;

	    fx = m90_spriteram[offs+5] & 0x02;
	    fy = m90_spriteram[offs+1] & 0x80;

	    y_multi = 1 << ((m90_spriteram[offs+1] & 0x60) >> 5);
	    y -= 16 * y_multi;

	    for (i = 0;i < y_multi;i++)
		pdrawgfx(bitmap,Machine->gfx[1],
		    sprite + (fy ? y_multi-1 - i : i),
		    colour,
		    fx,fy,
		    x,y+i*16,
		    cliprect,TRANSPARENCY_PEN,0,
		    (colour & 0x08) ? 0x00 : 0x02);
	}
}


static void mark_sprite_colors( void )
{
	int i,offs,color,pal_base,colmask[16];
	int sprite_mask=(Machine->drv->gfxdecodeinfo[1].gfxlayout->total)-1;
	PEN_USAGE_DATATYPE *pen_usage; /* Save some struct derefs */
	pal_base = Machine->drv->gfxdecodeinfo[1].color_codes_start;
	pen_usage=Machine->gfx[1]->pen_usage;
	for (color = 0;color < 16;color++) colmask[color] = 0;

	for (offs = 0x1f2;offs >= 0;offs -= 6)
	{
	    int sprite,colour;

	    sprite = (m90_spriteram[offs+2] | (m90_spriteram[offs+3]<<8));
	    colour = (m90_spriteram[offs+1] >> 1) & 0x0f;

	    colmask[color] |= pen_usage[(sprite+0)&sprite_mask] | pen_usage[(sprite+1)&sprite_mask];
	    colmask[color] |= pen_usage[(sprite+2)&sprite_mask] | pen_usage[(sprite+3)&sprite_mask];
	    colmask[color] |= pen_usage[(sprite+4)&sprite_mask] | pen_usage[(sprite+5)&sprite_mask];
	    colmask[color] |= pen_usage[(sprite+6)&sprite_mask] | pen_usage[(sprite+7)&sprite_mask];
	}

	for (color = 0;color < 16;color++)
	{
	    for (i = 1;i < 16;i++)
	    {
		if (colmask[color] & (1 << i))
		    palette_used_colors[pal_base + 16 * color + i] = PALETTE_COLOR_USED;
	    }
	}
}

#else
static void mark_sprite_colors( void )
{
	int i,offs,color,pal_base,colmask[16];
	PEN_USAGE_DATATYPE *pen_usage; /* Save some struct derefs */

	int sprite_mask=(Machine->drv->gfxdecodeinfo[1].gfxlayout->total)-1;

	pal_base = Machine->drv->gfxdecodeinfo[1].color_codes_start;
	pen_usage=Machine->gfx[1]->pen_usage;
	for (color = 0;color < 16;color++) colmask[color] = 0;

	for (offs = 0x0;offs < 0x200-8; offs+=6)
	{
	    int sprite;

	    sprite=m90_spriteram[offs+2] | (m90_spriteram[offs+3]<<8);
	    color=(m90_spriteram[offs+1]>>1)&0xf;

	    colmask[color] |= pen_usage[(sprite+0)&sprite_mask] | pen_usage[(sprite+1)&sprite_mask];
	    colmask[color] |= pen_usage[(sprite+2)&sprite_mask] | pen_usage[(sprite+3)&sprite_mask];
	    colmask[color] |= pen_usage[(sprite+4)&sprite_mask] | pen_usage[(sprite+5)&sprite_mask];
	    colmask[color] |= pen_usage[(sprite+6)&sprite_mask] | pen_usage[(sprite+7)&sprite_mask];
	}

	for (color = 0;color < 16;color++)
	{
	    for (i = 1;i < 16;i++)
	    {
		if (colmask[color] & (1 << i))
		    palette_used_colors[pal_base + 16 * color + i] = PALETTE_COLOR_USED;
	    }
	}
}


static void m90_drawsprites(struct osd_bitmap *bitmap,const struct rectangle *cliprect)
{
	int offs;

	for (offs = 0x1f2;offs >= 0;offs -= 6)
	{
		int x,y,sprite,colour,fx,fy,y_multi,i;

		sprite = (m90_spriteram[offs+2] | (m90_spriteram[offs+3]<<8));
		colour = (m90_spriteram[offs+1] >> 1) & 0x0f;

		y = m90_spriteram[offs+0] | ((m90_spriteram[offs+1] & 0x01) << 8);
		x = m90_spriteram[offs+4] | ((m90_spriteram[offs+5] & 0x01) << 8);

		x = x - 16;
		y = 512 - y;

		fx = m90_spriteram[offs+5] & 0x02;
		fy = m90_spriteram[offs+1] & 0x80;

		y_multi = 1 << ((m90_spriteram[offs+1] & 0x60) >> 5);
		y -= (y_multi << 4);

		for (i = 0;i < y_multi;i++)
			if (m90_video_control_data[0xe] & 0x01)
			    pdrawgfx(bitmap,Machine->gfx[1],
				    sprite + (fy ? y_multi-1 - i : i),
				    colour,
				    fx,fy,
				    x,y+(i<<4),
				    cliprect,TRANSPARENCY_PEN,0,
				    (colour & 0x08) ? 0x00 : 0x02);
			else if (m90_video_control_data[0xe] & 0x02)
			    pdrawgfx(bitmap,Machine->gfx[1],
				    sprite + (fy ? y_multi-1 - i : i),
				    colour,
				    fx,fy,
				    x,y+(i<<4),
				    cliprect,TRANSPARENCY_PEN,0,
				    ((colour & 0x0c)==0x0c) ? 0x00 : 0x02);
			else
			    pdrawgfx(bitmap,Machine->gfx[1],
				    sprite + (fy ? y_multi-1 - i : i),
				    colour,
				    fx,fy,
				    x,y+(i<<4),
				    cliprect,TRANSPARENCY_PEN,0,
				    0x02);
	}
}
#endif



WRITE_HANDLER( m90_video_control_w )
{
	m90_video_control_data[offset]=data;

	if (offset == 0xc)
	{
	    int new_pf2_size = (m90_video_control_data[0xc] & 0x4) >> 2; // pf2 == wide
	    if (new_pf2_size != pf2_size)
	    {
		if (pf2_layer)
		    tilemap_dispose(pf2_layer);
		switch (new_pf2_size)
		{
		    case 0: pf2_layer = tilemap_create(get_pf2_tile_info, tilemap_scan_rows,TILEMAP_OPAQUE,8,8,64,64); break;
		    case 1: pf2_layer = tilemap_create(get_pf2w_tile_info,tilemap_scan_rows,TILEMAP_OPAQUE,8,8,128,64); break;
		}
		pf2_size = new_pf2_size;
	    }
	}
	else if (offset == 0xa)
	{
	    int new_pf1_size = (m90_video_control_data[0xa] & 0x4) >> 2; // pf1 == wide
	    if (new_pf1_size != pf1_size)
	    {
		if (pf1_layer)
		    tilemap_dispose(pf1_layer);
		switch (new_pf1_size)
		{
		    case 0: pf1_layer = tilemap_create(get_pf1_tile_info, tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,64,64); break;
		    case 1: pf1_layer = tilemap_create(get_pf1w_tile_info,tilemap_scan_rows,TILEMAP_TRANSPARENT,8,8,128,64); break;
		}
		pf1_size = new_pf1_size;
		tilemap_set_transparent_pen(pf1_layer,0);
	    }
	}
}


inline static void markdirty(struct tilemap *tilemap,int page,offs_t offset)
{
	offset -= page * 0x4000;

	if (offset >= 0 && offset < 0x4000)
		tilemap_mark_tile_dirty(tilemap,offset>>2);
}


WRITE_HANDLER( m90_video_w )
{
	m90_video_data[offset] = data;


	if (pf1_size == 0)
	    markdirty(pf1_layer,     m90_video_control_data[0xa] & 0x3,offset);
	else
	    //markdirty(pf1_wide_layer,m90_video_control_data[0xa] & 0x2,offset);
	    markdirty(pf1_layer,m90_video_control_data[0xa] & 0x2,offset);

	if (pf2_size == 0)
	    markdirty(pf2_layer,     m90_video_control_data[0xc] & 0x3,offset);
	else
	    //markdirty(pf2_wide_layer,m90_video_control_data[0xc] & 0x2,offset);
	    markdirty(pf2_layer,m90_video_control_data[0xc] & 0x2,offset);
}



void m90_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
	const struct rectangle *cliprect = &Machine->visible_area;

	static int last_pf1,last_pf2;
	int pf1_base = m90_video_control_data[0xa] & 0x3;
	int pf2_base = m90_video_control_data[0xc] & 0x3;
	int i,pf1_enable,pf2_enable;

	if (m90_video_control_data[0xa]&0x10) pf1_enable=0; else pf1_enable=1;
	if (m90_video_control_data[0xc]&0x10) pf2_enable=0; else pf2_enable=1;

	/* Dirty tilemaps if VRAM base changes */
	if (pf1_base!=last_pf1)
	{
		tilemap_mark_all_tiles_dirty(pf1_layer);
		//tilemap_mark_all_tiles_dirty(pf1_wide_layer);
	}
	if (pf2_base!=last_pf2)
	{
		tilemap_mark_all_tiles_dirty(pf2_layer);
		//tilemap_mark_all_tiles_dirty(pf2_wide_layer);
	}
	last_pf1=pf1_base;
	last_pf2=pf2_base;

	m90_spriteram=m90_video_data+0xee00;

	/* Setup scrolling */
	if (m90_video_control_data[0xa]&0x20)
	{
		tilemap_set_scroll_rows(pf1_layer,512);
		//tilemap_set_scroll_rows(pf1_wide_layer,512);
		if (pf1_size == 0)
		{
		    for (i=0; i<1024; i+=2)
			    tilemap_set_scrollx( pf1_layer,i>>1, (m90_video_data[0xf000+i]+(m90_video_data[0xf001+i]<<8))+2);
		}
		else
		{
		    for (i=0; i<1024; i+=2)
			    //tilemap_set_scrollx( pf1_wide_layer,i/2, (m90_video_data[0xf000+i]+(m90_video_data[0xf001+i]<<8))+256+2);
			    tilemap_set_scrollx( pf1_layer,i>>1, (m90_video_data[0xf000+i]+(m90_video_data[0xf001+i]<<8))+256+2);
		}
	}
	else
	{
		tilemap_set_scroll_rows(pf1_layer,1);
		//tilemap_set_scroll_rows(pf1_wide_layer,1);
		if (pf1_size == 0)
		    tilemap_set_scrollx( pf1_layer,0, (m90_video_control_data[3]<<8)+m90_video_control_data[2]+2);
		else
		    //tilemap_set_scrollx( pf1_wide_layer,0, (m90_video_control_data[3]<<8)+m90_video_control_data[2]+256+2);
		    tilemap_set_scrollx( pf1_layer,0, (m90_video_control_data[3]<<8)+m90_video_control_data[2]+256+2);
	}

	/* Setup scrolling */
	if (m90_video_control_data[0xc]&0x20) {
		tilemap_set_scroll_rows(pf2_layer,512);
		//tilemap_set_scroll_rows(pf2_wide_layer,512);
		if (pf2_size == 0)
		{
		    for (i=0; i<1024; i+=2)
			    tilemap_set_scrollx( pf2_layer,i>>1, (m90_video_data[0xf400+i]+(m90_video_data[0xf401+i]<<8))-2);
		}
		else
		{
		    for (i=0; i<1024; i+=2)
			    //tilemap_set_scrollx( pf2_wide_layer,i/2, (m90_video_data[0xf400+i]+(m90_video_data[0xf401+i]<<8))+256-2);
			    tilemap_set_scrollx( pf2_layer,i>>1, (m90_video_data[0xf400+i]+(m90_video_data[0xf401+i]<<8))+256-2);
		}
	} else {
		tilemap_set_scroll_rows(pf2_layer,1);
		//tilemap_set_scroll_rows(pf2_wide_layer,1);
		if (pf2_size == 0)
		    tilemap_set_scrollx( pf2_layer,0, (m90_video_control_data[7]<<8)+m90_video_control_data[6]-2);
		else
		    //tilemap_set_scrollx( pf2_wide_layer,0, (m90_video_control_data[7]<<8)+m90_video_control_data[6]+256-2 );
		    tilemap_set_scrollx( pf2_layer,0, (m90_video_control_data[7]<<8)+m90_video_control_data[6]+256-2 );
	}

	tilemap_set_scrolly( pf1_layer,0, (m90_video_control_data[1]<<8)+m90_video_control_data[0] );
	tilemap_set_scrolly( pf2_layer,0, (m90_video_control_data[5]<<8)+m90_video_control_data[4] );
	//tilemap_set_scrolly( pf1_wide_layer,0, (m90_video_control_data[1]<<8)+m90_video_control_data[0] );
	//tilemap_set_scrolly( pf2_wide_layer,0, (m90_video_control_data[5]<<8)+m90_video_control_data[4] );

	tilemap_update(ALL_TILEMAPS);
	tilemap_render(ALL_TILEMAPS);
	palette_init_used_colors();
	mark_sprite_colors();
	palette_recalc();

	fillbitmap(priority_bitmap,0,cliprect);

	if (!pf2_enable)
		fillbitmap(bitmap,Machine->pens[0],cliprect);

	if (pf2_enable)
	{
#if 0
		if (m90_video_control_data[0xc] & 0x4)
			tilemap_draw(bitmap,pf2_wide_layer,0);
		else
			tilemap_draw(bitmap,pf2_layer,0);

		if (m90_video_control_data[0xc] & 0x4)
			tilemap_draw(bitmap,pf2_wide_layer,1);
		else
			tilemap_draw(bitmap,pf2_layer,1);
#else
		tilemap_draw(bitmap,pf2_layer,0);
		tilemap_draw(bitmap,pf2_layer,1|1<<16);
#endif
	}

	if (pf1_enable)
	{
#if 0
		if (m90_video_control_data[0xa] & 0x4)
			tilemap_draw(bitmap,pf1_wide_layer,0);
		else
			tilemap_draw(bitmap,pf1_layer,0);

		if (m90_video_control_data[0xa] & 0x4)
			tilemap_draw(bitmap,pf1_wide_layer,1);
		else
			tilemap_draw(bitmap,pf1_layer,1);
#else
		tilemap_draw(bitmap,pf1_layer,0);
		tilemap_draw(bitmap,pf1_layer,1|1<<16);
#endif
	}

	m90_drawsprites(bitmap,cliprect);
}
